@@ -4,7 +4,7 @@
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: atrhandler.h 3260 2009-01-02 15:19:59Z rousseau $
+ * $Id: atrhandler.h 4291 2009-07-01 12:23:22Z rousseau $
  */
 
 /**
@@ -29,9 +29,9 @@ extern "C"
 
 		struct _ATR
 		{
-			DWORD Length;
+			int Length;
+			int HistoryLength;
 			UCHAR Value[MAX_ATR_SIZE];
-			DWORD HistoryLength;
 			UCHAR HistoryValue[MAX_ATR_SIZE];
 		}
 		ATR;
