@@ -6,7 +6,7 @@
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: configfile.h 2331 2007-01-11 16:54:51Z rousseau $
+ * $Id: configfile.h 3260 2009-01-02 15:19:59Z rousseau $
  */
 
 #ifndef __configfile_h__
@@ -17,7 +17,8 @@ extern "C"
 {
 #endif
 
-	int DBGetReaderList(const char *readerconf, SerialReader **caller_reader_list);
+	int DBGetReaderList(const char *readerconf,
+		/*@out@*/ SerialReader **caller_reader_list);
 
 #ifdef __cplusplus
 }
