@@ -4,7 +4,7 @@
  * Copyright (C) 2008
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: hotplug_libhal.c 3304 2009-02-06 08:46:19Z rousseau $
+ * $Id: hotplug_libhal.c 4297 2009-07-03 12:55:47Z rousseau $
  */
 
 /**
@@ -31,9 +31,11 @@
 #include "hotplug.h"
 #include "thread_generic.h"
 #include "utils.h"
+#include "strlcpycat.h"
 
 #undef DEBUG_HOTPLUG
 #define ADD_SERIAL_NUMBER
+#define ADD_INTERFACE_NAME
 
 #define FALSE			0
 #define TRUE			1
@@ -61,6 +63,7 @@ static struct _driverTracker
 	char *libraryPath;
 	char *readerName;
 	int ifdCapabilities;
+	char *CFBundleName;
 } *driverTracker = NULL;
 #define DRIVER_TRACKER_SIZE_STEP 8
 
@@ -176,6 +179,12 @@ static LONG HPReadBundleValues(void)
 					driverTracker[listCount].ifdCapabilities = strtol(keyValue,
 						NULL, 16);
 
+				/* Get CFBundleName */
+				rv = LTPBundleFindOptionalValueWithKey(fullPath,
+					PCSCLITE_HP_CFBUNDLE_NAME, keyValue, 0);
+				if (0 == rv)
+					driverTracker[listCount].CFBundleName = strdup(keyValue);
+
 #ifdef DEBUG_HOTPLUG
 				Log2(PCSC_LOG_INFO, "Found driver for: %s",
 					driverTracker[listCount].readerName);
@@ -214,6 +223,7 @@ static LONG HPReadBundleValues(void)
 						driverTracker[i].libraryPath = NULL;
 						driverTracker[i].readerName = NULL;
 						driverTracker[i].ifdCapabilities = 0;
+						driverTracker[i].CFBundleName = NULL;
 					}
 				}
 			}
@@ -276,6 +286,7 @@ LONG HPStopHotPluggables(void)
 	DBusError error;
 	int i;
 	unsigned int idVendor, idProduct;
+	static struct _driverTracker *classdriver, *driver;
 
 	if (!libhal_device_property_exists(ctx, udi, "usb.vendor_id", NULL))
 		return NULL;
@@ -306,6 +317,8 @@ LONG HPStopHotPluggables(void)
 
 	Log3(PCSC_LOG_DEBUG, "Looking a driver for VID: 0x%04X, PID: 0x%04X", idVendor, idProduct);
 
+	classdriver = NULL;
+	driver = NULL;
 	/* check if the device is supported by one driver */
 	for (i=0; i<driverSize; i++)
 	{
@@ -313,11 +326,21 @@ LONG HPStopHotPluggables(void)
 			idVendor == driverTracker[i].manuID &&
 			idProduct == driverTracker[i].productID)
 		{
-			return &driverTracker[i];
+			if ((driverTracker[i].CFBundleName != NULL)
+				&& (0 == strcmp(driverTracker[i].CFBundleName, "CCIDCLASSDRIVER")))
+				classdriver = &driverTracker[i];
+			else
+				/* it is not a CCID Class driver */
+				driver = &driverTracker[i];
 		}
 	}
 
-	return NULL;
+	/* if we found a specific driver */
+	if (driver)
+		return driver;
+
+	/* else return the Class driver */
+	return classdriver;
 }
 
 
@@ -325,8 +348,9 @@ static void HPAddDevice(LibHalContext *ctx, const char *udi)
 {
 	int i;
 	char deviceName[MAX_DEVICENAME];
-	DBusError error;
 	struct _driverTracker *driver;
+	char *sSerialNumber = NULL, *sInterfaceName = NULL;
+	char fullname[MAX_READERNAME];
 	LONG ret;
 
 	driver = get_driver(ctx, udi);
@@ -367,40 +391,88 @@ static void HPAddDevice(LibHalContext *ctx, const char *udi)
 
 	readerTracker[i].udi = strdup(udi);
 
+#ifdef ADD_INTERFACE_NAME
+	if (libhal_device_property_exists(ctx, udi, "usb.interface.description", NULL))
+		sInterfaceName = libhal_device_get_property_string(ctx, udi,
+			"usb.interface.description", NULL);
+#endif
+
 #ifdef ADD_SERIAL_NUMBER
-	dbus_error_init (&error);
-
-	if (libhal_device_property_exists(ctx, udi, "usb.serial", &error))
-	{
-		char fullname[MAX_READERNAME];
-		char *sSerialNumber;
-
+	if (libhal_device_property_exists(ctx, udi, "usb.serial", NULL))
 		sSerialNumber = libhal_device_get_property_string(ctx, udi,
-			"usb.serial", &error);
+			"usb.serial", NULL);
+#endif
+	
+	/* name from the Info.plist file */
+	strlcpy(fullname, driver->readerName, sizeof(fullname));
 
-		(void)snprintf(fullname, sizeof(fullname), "%s (%s)",
-			driver->readerName, sSerialNumber);
-		readerTracker[i].fullName = strdup(fullname);
+	/* interface name from the device (if any) */
+	if (sInterfaceName)
+	{
+		strlcat(fullname, " [", sizeof(fullname));
+		strlcat(fullname, sInterfaceName, sizeof(fullname));
+		strlcat(fullname, "]", sizeof(fullname));
 	}
-	else
-#endif
-		readerTracker[i].fullName = strdup(driver->readerName);
-#ifdef ADD_SERIAL_NUMBER
-	if (dbus_error_is_set(&error))
-		dbus_error_free(&error);
-#endif
+
+	/* serial number from the device (if any) */
+	if (sSerialNumber)
+	{
+		strlcat(fullname, " (", sizeof(fullname));
+		strlcat(fullname, sSerialNumber, sizeof(fullname));
+		strlcat(fullname, ")", sizeof(fullname));
+	}
+
+	readerTracker[i].fullName = strdup(fullname);
 
 	ret = RFAddReader(readerTracker[i].fullName, PCSCLITE_HP_BASE_PORT + i,
 		driver->libraryPath, deviceName);
-	if (SCARD_S_SUCCESS != ret)
+	if ((SCARD_S_SUCCESS != ret) && (SCARD_E_UNKNOWN_READER != ret))
 	{
-		Log2(PCSC_LOG_ERROR, "Failed adding USB device: %s", short_name(udi));
-		free(readerTracker[i].fullName);
-		readerTracker[i].fullName = NULL;
-		free(readerTracker[i].udi);
-		readerTracker[i].udi = NULL;
+		char *parent, *device_file;
 
-		(void)CheckForOpenCT();
+		/* get the parent descriptor, without the '_if0' */
+		parent = libhal_device_get_property_string(ctx, udi,
+			"info.parent", NULL);
+		if (! parent)
+			goto error;
+
+		/* get the linux device file: i.e. '/dev/bus/usb/002/012' */
+		device_file = libhal_device_get_property_string(ctx, parent,
+			"linux.device_file", NULL);
+		if (! device_file)
+			goto error;
+
+		/* check the format looks correct */
+#define LIBUSB_HEADER "/dev/bus/usb/"
+		if (strncmp(device_file, LIBUSB_HEADER, strlen(LIBUSB_HEADER)))
+			goto error;
+
+		device_file += strlen(LIBUSB_HEADER);
+
+		(void)snprintf(deviceName, sizeof(deviceName),
+			"usb:%04x/%04x:libusb:%s",
+			driver->manuID, driver->productID, device_file);
+		deviceName[sizeof(deviceName) -1] = '\0';
+
+		/* replace the libusb separator '/' by ':' */
+		if ('/' == deviceName[strlen(deviceName)-3-1])
+			deviceName[strlen(deviceName)-3-1] = ':';
+
+		Log2(PCSC_LOG_INFO, "trying libusb scheme with: %s", deviceName);
+		ret = RFAddReader(readerTracker[i].fullName, PCSCLITE_HP_BASE_PORT + i,
+			driver->libraryPath, deviceName);
+
+		if (SCARD_S_SUCCESS != ret)
+		{
+error:
+			Log2(PCSC_LOG_ERROR, "Failed adding USB device: %s", short_name(udi));
+			free(readerTracker[i].fullName);
+			readerTracker[i].fullName = NULL;
+			free(readerTracker[i].udi);
+			readerTracker[i].udi = NULL;
+
+			(void)CheckForOpenCT();
+		}
 	}
 
 	(void)SYS_MutexUnLock(&usbNotifierMutex);
