@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdwrapper.c 3294 2009-01-28 13:34:28Z rousseau $
+ * $Id: ifdwrapper.c 4222 2009-05-22 11:39:14Z rousseau $
  */
 
 /**
@@ -636,7 +636,7 @@ LONG IFDControl(PREADER_CONTEXT rContext, DWORD ControlCode,
 	else
 	{
 		Log2(PCSC_LOG_ERROR, "Card not transacted: %ld", rv);
-		Log3(PCSC_LOG_DEBUG, "ControlCode: 0x%.8Lx BytesReturned: %ld",
+		Log3(PCSC_LOG_DEBUG, "ControlCode: 0x%.8LX BytesReturned: %ld",
 			ControlCode, *BytesReturned);
 		LogXxd(PCSC_LOG_DEBUG, "TxBuffer ", TxBuffer, TxLength);
 		LogXxd(PCSC_LOG_DEBUG, "RxBuffer ", RxBuffer, *BytesReturned);
