@@ -148,7 +148,7 @@ check_types (void)
     BLANK_LINE ();
     CHECK_STRUCT (establish_struct);
     CHECK_MEMBER (establish_struct, dwScope);
-    CHECK_MEMBER (establish_struct, phContext);
+    CHECK_MEMBER (establish_struct, hContext);
     CHECK_MEMBER (establish_struct, rv);
 
     BLANK_LINE ();
@@ -162,8 +162,8 @@ check_types (void)
     CHECK_MEMBER (connect_struct, szReader);
     CHECK_MEMBER (connect_struct, dwShareMode);
     CHECK_MEMBER (connect_struct, dwPreferredProtocols);
-    CHECK_MEMBER (connect_struct, phCard);
-    CHECK_MEMBER (connect_struct, pdwActiveProtocol);
+    CHECK_MEMBER (connect_struct, hCard);
+    CHECK_MEMBER (connect_struct, dwActiveProtocol);
     CHECK_MEMBER (connect_struct, rv);
 
     BLANK_LINE ();
@@ -172,7 +172,7 @@ check_types (void)
     CHECK_MEMBER (reconnect_struct, dwShareMode);
     CHECK_MEMBER (reconnect_struct, dwPreferredProtocols);
     CHECK_MEMBER (reconnect_struct, dwInitialization);
-    CHECK_MEMBER (reconnect_struct, pdwActiveProtocol);
+    CHECK_MEMBER (reconnect_struct, dwActiveProtocol);
     CHECK_MEMBER (reconnect_struct, rv);
 
     BLANK_LINE ();
@@ -202,8 +202,8 @@ check_types (void)
     CHECK_MEMBER (status_struct, hCard);
     CHECK_MEMBER (status_struct, mszReaderNames);
     CHECK_MEMBER (status_struct, pcchReaderLen);
-    CHECK_MEMBER (status_struct, pdwState);
-    CHECK_MEMBER (status_struct, pdwProtocol);
+    CHECK_MEMBER (status_struct, dwState);
+    CHECK_MEMBER (status_struct, dwProtocol);
     CHECK_MEMBER (status_struct, pbAtr);
     CHECK_MEMBER (status_struct, pcbAtrLen);
     CHECK_MEMBER (status_struct, rv);
@@ -211,12 +211,12 @@ check_types (void)
     BLANK_LINE ();
     CHECK_STRUCT (transmit_struct);
     CHECK_MEMBER (transmit_struct, hCard);
-    CHECK_MEMBER (transmit_struct, pioSendPciProtocol);
-    CHECK_MEMBER (transmit_struct, pioSendPciLength);
+    CHECK_MEMBER (transmit_struct, ioSendPciProtocol);
+    CHECK_MEMBER (transmit_struct, ioSendPciLength);
     CHECK_MEMBER (transmit_struct, pbSendBuffer);
     CHECK_MEMBER (transmit_struct, cbSendLength);
-    CHECK_MEMBER (transmit_struct, pioRecvPciProtocol);
-    CHECK_MEMBER (transmit_struct, pioRecvPciLength);
+    CHECK_MEMBER (transmit_struct, ioRecvPciProtocol);
+    CHECK_MEMBER (transmit_struct, ioRecvPciLength);
     CHECK_MEMBER (transmit_struct, pbRecvBuffer);
     CHECK_MEMBER (transmit_struct, pcbRecvLength);
     CHECK_MEMBER (transmit_struct, rv);
