@@ -6,7 +6,7 @@
  * Copyright (C) 1999-2008
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 3304 2009-02-06 08:46:19Z rousseau $
+ * $Id: pcscdaemon.c 4298 2009-07-03 13:11:44Z rousseau $
  */
 
 /**
@@ -396,14 +396,20 @@ int main(int argc, char **argv)
 	rv = SYS_Stat(PCSCLITE_IPC_DIR, &fStatBuf);
 	if (rv < 0)
 	{
-		rv = SYS_Mkdir(PCSCLITE_IPC_DIR,
-			S_IROTH | S_IXOTH | S_IRGRP | S_IXGRP | S_IRWXU);
+		int mode = S_IROTH | S_IXOTH | S_IRGRP | S_IXGRP | S_IRWXU;
+
+		rv = SYS_Mkdir(PCSCLITE_IPC_DIR, mode);
 		if (rv != 0)
 		{
 			Log2(PCSC_LOG_CRITICAL,
 				"cannot create " PCSCLITE_IPC_DIR ": %s", strerror(errno));
 			return EXIT_FAILURE;
 		}
+
+		/* set mode so that the directory is world readable and
+		 * executable even is umask is restrictive
+		 * The directory containes files used by libpcsclite */
+		(void)SYS_Chmod(PCSCLITE_IPC_DIR, mode);
 	}
 
 	/*
@@ -438,7 +444,8 @@ int main(int argc, char **argv)
 	rv = SYS_Stat(PCSCLITE_EVENTS_DIR, &fStatBuf);
 	if (rv < 0)
 	{
-		int mode = S_IRWXU | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH; /* 0755 */
+		/* 1733 : world writable + sticky bit */
+		int mode = S_IRWXU | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH | S_ISVTX;
 
 		rv = SYS_Mkdir(PCSCLITE_EVENTS_DIR, mode);
 		if (rv != 0)
@@ -554,6 +561,7 @@ static void clean_temp_files(void)
 			strerror(errno));
 
 	(void)StatSynchronize(NULL);
+	SYS_Sleep(1);
 	rv = SYS_RemoveFile(PCSCLITE_EVENTS_DIR);
 	if (rv != 0)
 		Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_EVENTS_DIR ": %s",
