@@ -7,7 +7,7 @@
  * Copyright (C) 2000-2008
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: thread_unix.c 2962 2008-05-16 09:32:19Z rousseau $
+ * $Id: thread_unix.c 3247 2009-01-02 13:22:46Z rousseau $
  */
 
 /**
@@ -74,7 +74,7 @@ INTERNAL int SYS_ThreadCreate(PCSCLITE_THREAD_T * pthThread, int attributes,
 		attributes & THREAD_ATTR_DETACHED ? PTHREAD_CREATE_DETACHED : PTHREAD_CREATE_JOINABLE);
 	if (ret)
 	{
-		pthread_attr_destroy(&attr);
+		(void)pthread_attr_destroy(&attr);
 		return ret;
 	}
 
