@@ -5,7 +5,7 @@
  *  David Corcoran <corcoran@linuxnet.com>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard.c 3467 2009-04-27 15:10:48Z rousseau $
+ * $Id: winscard.c 4332 2009-07-21 10:21:01Z rousseau $
  */
 
 /**
@@ -434,6 +434,8 @@ LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 					return SCARD_E_PROTO_MISMATCH;
 			}
 		}
+		else
+			rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 	}
 
 	*pdwActiveProtocol = rContext->readerState->cardProtocol;
@@ -762,6 +764,8 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 					return SCARD_E_PROTO_MISMATCH;
 			}
 		}
+		else
+			rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 	}
 
 	*pdwActiveProtocol = rContext->readerState->cardProtocol;
@@ -1669,7 +1673,7 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	{
 		*pcbRecvLength = 0;
 		Log2(PCSC_LOG_ERROR, "Card not transacted: 0x%08lX", rv);
-		return SCARD_E_NOT_TRANSACTED;
+		return rv;
 	}
 
 	/*
