@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.h 3260 2009-01-02 15:19:59Z rousseau $
+ * $Id: winscard_msg.h 4321 2009-07-17 14:21:50Z rousseau $
  */
 
 /**
@@ -30,6 +30,17 @@ extern "C"
 {
 #endif
 
+	/**
+	 * @brief Information transmitted in \ref CMD_VERSION Messages.
+	 */
+	struct version_struct
+	{
+		int32_t major;	/**< IPC major \ref PROTOCOL_VERSION_MAJOR */
+		int32_t minor;	/**< IPC minor \ref PROTOCOL_VERSION_MINOR */
+		uint32_t rv;
+	};
+	typedef struct version_struct version_struct;
+
 	/**
 	 * @brief General structure for client/serve message data exchange.
 	 *
@@ -49,7 +60,11 @@ extern "C"
 		uint32_t command;	/** one of the \c pcsc_msg_commands */
 		uint64_t date;
 		unsigned char key[PCSCLITE_MSG_KEY_LEN]; /* 16 bytes */
-		unsigned char data[PCSCLITE_MAX_MESSAGE_SIZE];
+		union
+		{
+			unsigned char data[PCSCLITE_MAX_MESSAGE_SIZE];
+			struct version_struct veStr;
+		};
 	}
 	sharedSegmentMsg, *psharedSegmentMsg;
 
@@ -93,17 +108,6 @@ extern "C"
 		SCARD_CONTROL_EXTENDED = 0x12	/**< used by SCardControl() */
 	};
 
-	/**
-	 * @brief Information transmitted in \ref CMD_VERSION Messages.
-	 */
-	struct version_struct
-	{
-		int32_t major;	/**< IPC major \ref PROTOCOL_VERSION_MAJOR */
-		int32_t minor;	/**< IPC minor \ref PROTOCOL_VERSION_MINOR */
-		uint32_t rv;
-	};
-	typedef struct version_struct version_struct;
-
 	struct client_struct
 	{
 		uint32_t hContext;
@@ -118,7 +122,7 @@ extern "C"
 	struct establish_struct
 	{
 		uint32_t dwScope;
-		uint32_t phContext;
+		uint32_t hContext;
 		uint32_t rv;
 	};
 	typedef struct establish_struct establish_struct;
@@ -146,8 +150,8 @@ extern "C"
 		char szReader[MAX_READERNAME];
 		uint32_t dwShareMode;
 		uint32_t dwPreferredProtocols;
-		int32_t phCard;
-		uint32_t pdwActiveProtocol;
+		int32_t hCard;
+		uint32_t dwActiveProtocol;
 		uint32_t rv;
 	};
 	typedef struct connect_struct connect_struct;
@@ -163,7 +167,7 @@ extern "C"
 		uint32_t dwShareMode;
 		uint32_t dwPreferredProtocols;
 		uint32_t dwInitialization;
-		uint32_t pdwActiveProtocol;
+		uint32_t dwActiveProtocol;
 		uint32_t rv;
 	};
 	typedef struct reconnect_struct reconnect_struct;
@@ -228,8 +232,8 @@ extern "C"
 		int32_t hCard;
 		char mszReaderNames[MAX_READERNAME];
 		uint32_t pcchReaderLen;
-		uint32_t pdwState;
-		uint32_t pdwProtocol;
+		uint32_t dwState;
+		uint32_t dwProtocol;
 		uint8_t pbAtr[MAX_ATR_SIZE];
 		uint32_t pcbAtrLen;
 		uint32_t rv;
@@ -244,12 +248,12 @@ extern "C"
 	struct transmit_struct
 	{
 		int32_t hCard;
-		uint32_t pioSendPciProtocol;
-		uint32_t pioSendPciLength;
+		uint32_t ioSendPciProtocol;
+		uint32_t ioSendPciLength;
 		uint8_t pbSendBuffer[MAX_BUFFER_SIZE];
 		uint32_t cbSendLength;
-		uint32_t pioRecvPciProtocol;
-		uint32_t pioRecvPciLength;
+		uint32_t ioRecvPciProtocol;
+		uint32_t ioRecvPciLength;
 		uint8_t pbRecvBuffer[MAX_BUFFER_SIZE];
 		uint32_t pcbRecvLength;
 		uint32_t rv;
@@ -264,11 +268,11 @@ extern "C"
 	struct transmit_struct_extended
 	{
 		int32_t hCard;
-		uint32_t pioSendPciProtocol;
-		uint32_t pioSendPciLength;
+		uint32_t ioSendPciProtocol;
+		uint32_t ioSendPciLength;
 		uint32_t cbSendLength;
-		uint32_t pioRecvPciProtocol;
-		uint32_t pioRecvPciLength;
+		uint32_t ioRecvPciProtocol;
+		uint32_t ioRecvPciLength;
 		uint32_t pcbRecvLength;
 		uint32_t rv;
 		uint64_t size;
@@ -305,7 +309,7 @@ extern "C"
 		uint32_t dwControlCode;
 		uint32_t cbSendLength;
 		uint32_t cbRecvLength;
-		uint32_t pdwBytesReturned;
+		uint32_t dwBytesReturned;
 		uint32_t rv;
 		uint64_t size;
 		uint8_t data[1];
