@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludoic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg_srv.c 2961 2008-05-16 09:12:53Z rousseau $
+ * $Id: winscard_msg_srv.c 3260 2009-01-02 15:19:59Z rousseau $
  */
 
 /**
@@ -49,6 +49,29 @@ static int commonSocket = 0;
 extern char AraKiri;
 extern char ReCheckSerialReaders;
 
+static const char *CommandsText[] = {
+	"CMD_VERSION",	/* mtype = 0xF8 and command = 0x00 */
+	"ESTABLISH_CONTEXT",	/* mtype = 0xF1 */
+	"RELEASE_CONTEXT",
+	"LIST_READERS",
+	"CONNECT",
+	"RECONNECT",
+	"DISCONNECT",
+	"BEGIN_TRANSACTION",
+	"END_TRANSACTION",
+	"TRANSMIT",
+	"CONTROL",
+	"STATUS",
+	"GET_STATUS_CHANGE",
+	"CANCEL",
+	"CANCEL_TRANSACTION",
+	"GET_ATTRIB",
+	"SET_ATTRIB",
+	"TRANSMIT_EXTENDED",
+	"CONTROL_EXTENDED",
+	"NULL"
+};
+
 /**
  * @brief Accepts a Client connection.
  *
@@ -61,7 +84,7 @@ extern char ReCheckSerialReaders;
  * @retval -1 Can not establish the connection.
  * @retval -1 Can not set the connection to non-blocking mode.
  */
-static int SHMProcessCommonChannelRequest(uint32_t *pdwClientID)
+static int SHMProcessCommonChannelRequest(/*@out@*/ uint32_t *pdwClientID)
 {
 	socklen_t clnt_len;
 	int new_sock;
@@ -85,7 +108,7 @@ static int SHMProcessCommonChannelRequest(uint32_t *pdwClientID)
 	{
 		Log2(PCSC_LOG_CRITICAL, "Error: cannot set socket nonblocking: %s",
 			strerror(errno));
-		SYS_CloseFile(*pdwClientID);
+		(void)SYS_CloseFile(*pdwClientID);
 		*pdwClientID = -1;
 		return -1;
 	}
@@ -107,7 +130,7 @@ static int SHMProcessCommonChannelRequest(uint32_t *pdwClientID)
  * @retval -1 Can not bind the socket to the file \c PCSCLITE_CSOCK_NAME.
  * @retval -1 Can not put the socket in listen mode.
  */
-INTERNAL int SHMInitializeCommonSegment(void)
+INTERNAL int32_t SHMInitializeCommonSegment(void)
 {
 	static struct sockaddr_un serv_adr;
 
@@ -124,7 +147,7 @@ INTERNAL int SHMInitializeCommonSegment(void)
 	serv_adr.sun_family = AF_UNIX;
 	strncpy(serv_adr.sun_path, PCSCLITE_CSOCK_NAME,
 		sizeof(serv_adr.sun_path));
-	SYS_Unlink(PCSCLITE_CSOCK_NAME);
+	(void)SYS_RemoveFile(PCSCLITE_CSOCK_NAME);
 
 	if (bind(commonSocket, (struct sockaddr *) &serv_adr,
 			sizeof(serv_adr.sun_family) + strlen(serv_adr.sun_path) + 1) < 0)
@@ -146,7 +169,7 @@ INTERNAL int SHMInitializeCommonSegment(void)
 	/*
 	 * Chmod the public entry channel
 	 */
-	SYS_Chmod(PCSCLITE_CSOCK_NAME, S_IRWXO | S_IRWXG | S_IRWXU);
+	(void)SYS_Chmod(PCSCLITE_CSOCK_NAME, S_IRWXO | S_IRWXG | S_IRWXU);
 
 	return 0;
 }
@@ -157,7 +180,6 @@ INTERNAL int SHMInitializeCommonSegment(void)
  * This is called by the Server's function \c SVCServiceRunLoop().
  *
  * @param[out] pdwClientID Connection ID used to reference the Client.
- * @param[in] blocktime Timeout (not used).
  *
  * @return Error code.
  * @retval 0 Success.
@@ -168,7 +190,7 @@ INTERNAL int SHMInitializeCommonSegment(void)
 #if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 #define DO_TIMEOUT
 #endif
-INTERNAL int32_t SHMProcessEventsServer(uint32_t *pdwClientID, int32_t blocktime)
+INTERNAL int32_t SHMProcessEventsServer(uint32_t *pdwClientID)
 {
 	fd_set read_fd;
 	int selret;
@@ -237,20 +259,28 @@ INTERNAL int32_t SHMProcessEventsServer(uint32_t *pdwClientID, int32_t blocktime
  * Called by \c ContextThread().
  */
 INTERNAL int32_t SHMProcessEventsContext(uint32_t dwClientID,
-	psharedSegmentMsg msgStruct, int32_t blocktime)
+	psharedSegmentMsg msgStruct)
 {
 	fd_set read_fd;
 	int selret, rv;
+#ifdef DO_TIMEOUT
 	struct timeval tv;
 
 	tv.tv_sec = 1;
 	tv.tv_usec = 0;
+#endif
 
 	FD_ZERO(&read_fd);
 	FD_SET(dwClientID, &read_fd);
 
 	selret = select(dwClientID + 1, &read_fd, (fd_set *) NULL,
-		(fd_set *) NULL, &tv);
+		(fd_set *) NULL,
+#ifdef DO_TIMEOUT
+		&tv
+#else
+		NULL
+#endif
+		);
 
 	if (selret < 0)
 	{
@@ -276,7 +306,7 @@ INTERNAL int32_t SHMProcessEventsContext(uint32_t dwClientID,
 			Log2(PCSC_LOG_DEBUG, "Client has disappeared: %d", dwClientID);
 			msgStruct->mtype = CMD_CLIENT_DIED;
 			msgStruct->command = 0;
-			SYS_CloseFile(dwClientID);
+			(void)SYS_CloseFile(dwClientID);
 
 			return 0;
 		}
@@ -284,7 +314,7 @@ INTERNAL int32_t SHMProcessEventsContext(uint32_t dwClientID,
 		/*
 		 * Set the identifier handle
 		 */
-		Log2(PCSC_LOG_DEBUG, "correctly processed client: %d", dwClientID);
+		Log3(PCSC_LOG_DEBUG, "command %s received by client %d", CommandsText[msgStruct->command], dwClientID);
 		return 1;
 	}
 
