@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_svc.c 3267 2009-01-05 13:58:14Z rousseau $
+ * $Id: winscard_svc.c 4334 2009-07-21 14:26:19Z rousseau $
  */
 
 /**
@@ -178,7 +178,7 @@ static void ContextThread(LPVOID dwIndex)
 				if (msgStruct.mtype == CMD_VERSION)
 				{
 					version_struct *veStr;
-					veStr = (version_struct *) msgStruct.data;
+					veStr = &msgStruct.veStr;
 
 					/* get the client protocol version */
 					psContext[dwContextIndex].protocol_major = veStr->major;
@@ -295,13 +295,13 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 	case SCARD_ESTABLISH_CONTEXT:
 		esStr = ((establish_struct *) msgStruct->data);
 
-		hContext = esStr->phContext;
+		hContext = esStr->hContext;
 		esStr->rv = SCardEstablishContext(esStr->dwScope, 0, 0, &hContext);
-		esStr->phContext = hContext;
+		esStr->hContext = hContext;
 
 		if (esStr->rv == SCARD_S_SUCCESS)
 			esStr->rv =
-				MSGAddContext(esStr->phContext, dwContextIndex);
+				MSGAddContext(esStr->hContext, dwContextIndex);
 		break;
 
 	case SCARD_RELEASE_CONTEXT:
@@ -317,19 +317,19 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 	case SCARD_CONNECT:
 		coStr = ((connect_struct *) msgStruct->data);
 
-		hCard = coStr->phCard;
-		dwActiveProtocol = coStr->pdwActiveProtocol;
+		hCard = coStr->hCard;
+		dwActiveProtocol = coStr->dwActiveProtocol;
 
 		coStr->rv = SCardConnect(coStr->hContext, coStr->szReader,
 			coStr->dwShareMode, coStr->dwPreferredProtocols,
 			&hCard, &dwActiveProtocol);
 
-		coStr->phCard = hCard;
-		coStr->pdwActiveProtocol = dwActiveProtocol;
+		coStr->hCard = hCard;
+		coStr->dwActiveProtocol = dwActiveProtocol;
 
 		if (coStr->rv == SCARD_S_SUCCESS)
 			coStr->rv =
-				MSGAddHandle(coStr->hContext, coStr->phCard, dwContextIndex);
+				MSGAddHandle(coStr->hContext, coStr->hCard, dwContextIndex);
 
 		break;
 
@@ -341,7 +341,7 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 		rcStr->rv = SCardReconnect(rcStr->hCard, rcStr->dwShareMode,
 			rcStr->dwPreferredProtocols,
 			rcStr->dwInitialization, &dwActiveProtocol);
-		rcStr->pdwActiveProtocol = dwActiveProtocol;
+		rcStr->dwActiveProtocol = dwActiveProtocol;
 		break;
 
 	case SCARD_DISCONNECT:
@@ -383,17 +383,25 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 		if (rv != 0) return rv;
 
 		cchReaderLen = stStr->pcchReaderLen;
-		dwState = stStr->pdwState;
-		dwProtocol = stStr->pdwProtocol;
+		dwState = stStr->dwState;
+		dwProtocol = stStr->dwProtocol;
 		cbAtrLen = stStr->pcbAtrLen;
 
+		/* avoids buffer overflow */
+		if ((cchReaderLen > sizeof(stStr->mszReaderNames))
+			|| (cbAtrLen > sizeof(stStr->pbAtr)))
+		{
+			stStr->rv = SCARD_E_INSUFFICIENT_BUFFER ;
+			break;
+		}
+
 		stStr->rv = SCardStatus(stStr->hCard, stStr->mszReaderNames,
 			&cchReaderLen, &dwState,
 			&dwProtocol, stStr->pbAtr, &cbAtrLen);
 
 		stStr->pcchReaderLen = cchReaderLen;
-		stStr->pdwState = dwState;
-		stStr->pdwProtocol = dwProtocol;
+		stStr->dwState = dwState;
+		stStr->dwProtocol = dwProtocol;
 		stStr->pcbAtrLen = cbAtrLen;
 		break;
 
@@ -402,10 +410,18 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 		rv = MSGCheckHandleAssociation(trStr->hCard, dwContextIndex);
 		if (rv != 0) return rv;
 
-		ioSendPci.dwProtocol = trStr->pioSendPciProtocol;
-		ioSendPci.cbPciLength = trStr->pioSendPciLength;
-		ioRecvPci.dwProtocol = trStr->pioRecvPciProtocol;
-		ioRecvPci.cbPciLength = trStr->pioRecvPciLength;
+		/* avoids buffer overflow */
+		if ((trStr->pcbRecvLength > sizeof(trStr->pbRecvBuffer))
+			|| (trStr->cbSendLength > sizeof(trStr->pbSendBuffer)))
+		{
+			trStr->rv = SCARD_E_INSUFFICIENT_BUFFER ;
+			break;
+		}
+
+		ioSendPci.dwProtocol = trStr->ioSendPciProtocol;
+		ioSendPci.cbPciLength = trStr->ioSendPciLength;
+		ioRecvPci.dwProtocol = trStr->ioRecvPciProtocol;
+		ioRecvPci.cbPciLength = trStr->ioRecvPciLength;
 		cbRecvLength = trStr->pcbRecvLength;
 
 		trStr->rv = SCardTransmit(trStr->hCard, &ioSendPci,
@@ -413,10 +429,10 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 			&ioRecvPci, trStr->pbRecvBuffer,
 			&cbRecvLength);
 
-		trStr->pioSendPciProtocol = ioSendPci.dwProtocol;
-		trStr->pioSendPciLength = ioSendPci.cbPciLength;
-		trStr->pioRecvPciProtocol = ioRecvPci.dwProtocol;
-		trStr->pioRecvPciLength = ioRecvPci.cbPciLength;
+		trStr->ioSendPciProtocol = ioSendPci.dwProtocol;
+		trStr->ioSendPciLength = ioSendPci.cbPciLength;
+		trStr->ioRecvPciProtocol = ioRecvPci.dwProtocol;
+		trStr->ioRecvPciLength = ioRecvPci.cbPciLength;
 		trStr->pcbRecvLength = cbRecvLength;
 
 		break;
@@ -426,6 +442,14 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 		rv = MSGCheckHandleAssociation(ctStr->hCard, dwContextIndex);
 		if (rv != 0) return rv;
 
+		/* avoids buffer overflow */
+		if ((ctStr->cbRecvLength > sizeof(ctStr->pbRecvBuffer))
+			|| (ctStr->cbSendLength > sizeof(ctStr->pbSendBuffer)))
+		{
+			ctStr->rv = SCARD_E_INSUFFICIENT_BUFFER;
+			break;
+		}
+
 		dwBytesReturned = ctStr->dwBytesReturned;
 
 		ctStr->rv = SCardControl(ctStr->hCard, ctStr->dwControlCode,
@@ -442,6 +466,13 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 		rv = MSGCheckHandleAssociation(gsStr->hCard, dwContextIndex);
 		if (rv != 0) return rv;
 
+		/* avoids buffer overflow */
+		if (gsStr->cbAttrLen > sizeof(gsStr->pbAttr))
+		{
+			gsStr->rv = SCARD_E_INSUFFICIENT_BUFFER ;
+			break;
+		}
+
 		cbAttrLen = gsStr->cbAttrLen;
 
 		gsStr->rv = SCardGetAttrib(gsStr->hCard, gsStr->dwAttrId,
@@ -455,6 +486,14 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 		gsStr = ((getset_struct *) msgStruct->data);
 		rv = MSGCheckHandleAssociation(gsStr->hCard, dwContextIndex);
 		if (rv != 0) return rv;
+
+		/* avoids buffer overflow */
+		if (gsStr->cbAttrLen <= sizeof(gsStr->pbAttr))
+		{
+			gsStr->rv = SCARD_E_INSUFFICIENT_BUFFER ;
+			break;
+		}
+
 		gsStr->rv = SCardSetAttrib(gsStr->hCard, gsStr->dwAttrId,
 			gsStr->pbAttr, gsStr->cbAttrLen);
 		break;
@@ -469,6 +508,15 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 			rv = MSGCheckHandleAssociation(treStr->hCard, dwContextIndex);
 			if (rv != 0) return rv;
 
+			/* avoids buffer overflow */
+			if ((treStr->size > sizeof(pbSendBuffer))
+				|| (treStr->cbSendLength > sizeof(pbSendBuffer))
+				|| (treStr->pcbRecvLength > sizeof(pbRecvBuffer)))
+			{
+				treStr->rv = SCARD_E_INSUFFICIENT_BUFFER;
+				break;
+			}
+
 			/* on more block to read? */
 			if (treStr->size > PCSCLITE_MAX_MESSAGE_SIZE)
 			{
@@ -488,10 +536,10 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 			else
 				memcpy(pbSendBuffer, treStr->data, treStr->cbSendLength);
 
-			ioSendPci.dwProtocol = treStr->pioSendPciProtocol;
-			ioSendPci.cbPciLength = treStr->pioSendPciLength;
-			ioRecvPci.dwProtocol = treStr->pioRecvPciProtocol;
-			ioRecvPci.cbPciLength = treStr->pioRecvPciLength;
+			ioSendPci.dwProtocol = treStr->ioSendPciProtocol;
+			ioSendPci.cbPciLength = treStr->ioSendPciLength;
+			ioRecvPci.dwProtocol = treStr->ioRecvPciProtocol;
+			ioRecvPci.cbPciLength = treStr->ioRecvPciLength;
 			cbRecvLength = treStr->pcbRecvLength;
 
 			treStr->rv = SCardTransmit(treStr->hCard, &ioSendPci,
@@ -499,10 +547,10 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 				&ioRecvPci, pbRecvBuffer,
 				&cbRecvLength);
 
-			treStr->pioSendPciProtocol = ioSendPci.dwProtocol;
-			treStr->pioSendPciLength = ioSendPci.cbPciLength;
-			treStr->pioRecvPciProtocol = ioRecvPci.dwProtocol;
-			treStr->pioRecvPciLength = ioRecvPci.cbPciLength;
+			treStr->ioSendPciProtocol = ioSendPci.dwProtocol;
+			treStr->ioSendPciLength = ioSendPci.cbPciLength;
+			treStr->ioRecvPciProtocol = ioRecvPci.dwProtocol;
+			treStr->ioRecvPciLength = ioRecvPci.cbPciLength;
 			treStr->pcbRecvLength = cbRecvLength;
 
 			treStr->size = offsetof(transmit_struct_extended, data) + treStr->pcbRecvLength;
@@ -550,6 +598,15 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 			rv = MSGCheckHandleAssociation(cteStr->hCard, dwContextIndex);
 			if (rv != 0) return rv;
 
+			/* avoids buffer overflow */
+			if ((cteStr->size > sizeof(pbSendBuffer))
+				|| (cteStr->cbSendLength > sizeof(pbSendBuffer))
+				|| (cteStr->cbRecvLength > sizeof(pbRecvBuffer)))
+			{
+				cteStr->rv = SCARD_E_INSUFFICIENT_BUFFER;
+				break;
+			}
+
 			/* on more block to read? */
 			if (cteStr->size > PCSCLITE_MAX_MESSAGE_SIZE)
 			{
@@ -569,16 +626,16 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 			else
 				memcpy(pbSendBuffer, cteStr->data, cteStr->cbSendLength);
 
-			dwBytesReturned = cteStr->pdwBytesReturned;
+			dwBytesReturned = cteStr->dwBytesReturned;
 
 			cteStr->rv = SCardControl(cteStr->hCard, cteStr->dwControlCode,
 				pbSendBuffer, cteStr->cbSendLength,
 				pbRecvBuffer, cteStr->cbRecvLength,
 				&dwBytesReturned);
 
-			cteStr->pdwBytesReturned = dwBytesReturned;
+			cteStr->dwBytesReturned = dwBytesReturned;
 
-			cteStr->size = sizeof(*cteStr) + cteStr->pdwBytesReturned;
+			cteStr->size = sizeof(*cteStr) + cteStr->dwBytesReturned;
 			if (cteStr->size > PCSCLITE_MAX_MESSAGE_SIZE)
 			{
 				/* two blocks */
@@ -602,7 +659,7 @@ static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
 			else
 			{
 				/* one block only */
-				memcpy(cteStr->data, pbRecvBuffer, cteStr->pdwBytesReturned);
+				memcpy(cteStr->data, pbRecvBuffer, cteStr->dwBytesReturned);
 
 				rv = SHMMessageSend(msgStruct, sizeof(*msgStruct),
 					psContext[dwContextIndex].dwClientID,
